#! /usr/bin/python3
# -*- coding: utf-8 -*-

# Copyright(C) 2013 name <email>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from gi.repository import GLib, Gio
from gi.repository import Unity
import gettext
import urllib.request
import json

APP_NAME = 'unity-scope-ebay'
LOCAL_PATH = '/usr/share/locale/'
gettext.bindtextdomain(APP_NAME, LOCAL_PATH)
gettext.textdomain(APP_NAME)
_ = gettext.gettext

GROUP_NAME = 'com.canonical.Unity.Scope.Shopping.Ebay'
UNIQUE_PATH = '/com/canonical/unity/scope/shopping/ebay'

SEARCH_HINT = _('Search ebay')
NO_RESULTS_HINT = _('Sorry, there are no ebay results that match your search.')
PROVIDER_CREDITS = _('Powered by ebay')
SVG_DIR = '/usr/share/icons/unity-icon-theme/places/svg/'
PROVIDER_ICON = SVG_DIR + 'service-ebay.svg'
DEFAULT_RESULT_ICON = SVG_DIR + 'result-shopping.svg'
DEFAULT_RESULT_MIMETYPE = 'text/html'
DEFAULT_RESULT_TYPE = Unity.ResultType.DEFAULT

SEARCH_URI = 'https://svcs.ebay.com/services/search/FindingService/v1?SECURITY-APPNAME=%s&OPERATION-NAME=findItemsByKeywords&SERVICE-VERSION=1.0.0&RESPONSE-DATA-FORMAT=JSON&REST-PAYLOAD&keywords=%s&paginationInput.entriesPerPage=10'
API_KEY = 'MarkTull-b296-4c1f-bad6-7d1e8661f16f'

c1 = {'id': 'items',
      'name': _('Items'),
      'icon': SVG_DIR + 'group-installed.svg',
      'renderer': Unity.CategoryRenderer.VERTICAL_TILE}
CATEGORIES = [c1]

FILTERS = []

EXTRA_METADATA = []


def search(search, filters):
    '''
    Search the server for questions and use stackexchange's api for tags
    '''
    results = []
    search_url = SEARCH_URI % (API_KEY, search.replace(" ", "%20"))
    try:
        response = urllib.request.urlopen(search_url).read()
        response = response.decode('utf8')
        json_response = json.loads(response)
        result = json_response["findItemsByKeywordsResponse"][0]['searchResult'][0]['item']
    except:
        result = {}
    if result:
        for item in result:

            icon_hint = Unity.AnnotatedIcon.new(Gio.ThemedIcon.new(item['galleryURL'][0]))
            icon_hint.props.ribbon = "%s %.2f" % (item['sellingStatus'][0]['currentPrice'][0]['@currencyId'],
                                                  float(item['sellingStatus'][0]['currentPrice'][0]['__value__']))
            icon_hint.props.category = Unity.CategoryType.HOME

            title = item['title'][0]
            uri = item['viewItemURL'][0]
            comment = "%s %.2f" % (item['sellingStatus'][0]['currentPrice'][0]['@currencyId'],
                                   float(item['sellingStatus'][0]['currentPrice'][0]['__value__']))
            results.append({'uri': uri,
                            'icon_hint': icon_hint.to_string(),
                            'category': 0,
                            'title': title,
                            'comment': comment})

    return results


# Classes below this point establish communication
# with Unity, you probably shouldn't modify them.


class MySearch(Unity.ScopeSearchBase):
    def __init__(self, search_context):
        super(MySearch, self).__init__()
        self.set_search_context(search_context)

    def do_run(self):
        '''
        Adds results to the model
        '''
        try:
            result_set = self.search_context.result_set
            for i in search(self.search_context.search_query,
                            self.search_context.filter_state):
                if not 'uri' in i or not i['uri'] or i['uri'] == '':
                    continue
                if not 'icon' in i or not i['icon'] or i['icon'] == '':
                    i['icon'] = DEFAULT_RESULT_ICON
                if not 'mimetype' in i or not i['mimetype'] or i['mimetype'] == '':
                    i['mimetype'] = DEFAULT_RESULT_MIMETYPE
                if not 'result_type' in i or not i['result_type'] or i['result_type'] == '':
                    i['result_type'] = DEFAULT_RESULT_TYPE
                if not 'category' in i or not i['category'] or i['category'] == '':
                    i['category'] = 0
                if not 'title' in i or not i['title']:
                    i['title'] = ''
                if not 'comment' in i or not i['comment']:
                    i['comment'] = ''
                if not 'dnd_uri' in i or not i['dnd_uri'] or i['dnd_uri'] == '':
                    i['dnd_uri'] = i['uri']
                i['metadata'] = {}
                if EXTRA_METADATA:
                    for e in i:
                        for m in EXTRA_METADATA:
                            if m['id'] == e:
                                i['metadata'][e] = i[e]
                i['metadata']['provider_credits'] = GLib.Variant('s', PROVIDER_CREDITS)
                result = Unity.ScopeResult.create(str(i['uri']), str(i['icon']),
                                                  i['category'], i['result_type'],
                                                  str(i['mimetype']), str(i['title']),
                                                  str(i['comment']), str(i['dnd_uri']),
                                                  i['metadata'])
                result_set.add_result(result)
        except Exception as error:
            print(error)


class Scope(Unity.AbstractScope):
    def __init__(self):
        Unity.AbstractScope.__init__(self)

    def do_get_search_hint(self):
        return SEARCH_HINT

    def do_get_schema(self):
        '''
        Adds specific metadata fields
        '''
        schema = Unity.Schema.new()
        if EXTRA_METADATA:
            for m in EXTRA_METADATA:
                schema.add_field(m['id'], m['type'], m['field'])
        #FIXME should be REQUIRED for credits
        schema.add_field('provider_credits', 's', Unity.SchemaFieldType.OPTIONAL)
        return schema

    def do_get_categories(self):
        '''
        Adds categories
        '''
        cs = Unity.CategorySet.new()
        if CATEGORIES:
            for c in CATEGORIES:
                cat = Unity.Category.new(c['id'], c['name'],
                                         Gio.ThemedIcon.new(c['icon']),
                                         c['renderer'])
                cs.add(cat)
        return cs

    def do_get_filters(self):
        '''
        Adds filters
        '''
        fs = Unity.FilterSet.new()
        #if FILTERS:
        #
        return fs

    def do_get_group_name(self):
        return GROUP_NAME

    def do_get_unique_name(self):
        return UNIQUE_PATH

    def do_create_search_for_query(self, search_context):
        se = MySearch(search_context)
        return se


def load_scope():
    return Scope()
